package tcbnet;

import tcbnet.control.Configuration;
import tcbnet.control.Identification;
import tcbnet.corba.Node;
import tcbnet.corba.NodeImpl;
import tcbnet.corba.TcbnetOrb;
import tcbnet.gui.MainWindow;
import tcbnet.ui.UIController;

import java.io.*;

/**
 * Classe de execucao primaria do TCBNet.
 *
 * @author $Author: dseki $
 * @version $Revision: 1.7 $
 */

public class TCBNet
{
    private static boolean running = true;
    private static Object runningLock = new Object();
    private static long REFRESH_TIME = 1500;

    /**
     * Programa principal.
     *
     * @param argv argumentos da linha de comando
     */
    public static void main(String argv[])
    {
	System.out.println("* Carregando configuracoes");
	Configuration configuration = Configuration.load();

	System.out.println("* Carregando no' de rede\n");
	NodeImpl nodeImpl = new NodeImpl(configuration);
	Node node = TcbnetOrb.getInstance().getNode(nodeImpl);

	try { Identification.generateNodeID(nodeImpl.getIor()); }
	catch (IOException ioe) {}

	System.out.println("\n* Carregando interface");
	UIController uiController = new UIController(configuration, nodeImpl);
	uiController.uiStart();

	System.out.println("* Pronto e rodando");
	synchronized (TCBNet.runningLock) {
	    while (TCBNet.running) {
		try {
		    TCBNet.runningLock.wait(TCBNet.REFRESH_TIME);
		}
		catch (InterruptedException ie) {
		    TCBNet.running = false;
		}
		uiController.refreshUI();
	    }
	}

	System.out.println("* Encerrando programa");
	try {
	    //uiController.saveConfiguration();
	    //configuration.setSeenNodes(nodeImpl.getP2PNodeList());
	    configuration.save();
	}
	catch (IOException ioe) {
	    System.out.println("Nao foi possivel salvar as configuracoes");
	}
    }

    /**
     * Interrompe o laco infinito do programa principal, iniciando o procedi-
     * mento de encerramento do programa.
     */
    public static void exit()
    {
	synchronized (TCBNet.runningLock) {
	    TCBNet.running = false;
	    TCBNet.runningLock.notify();
	}
    }
}
