package tcbnet.control;

/**
 * Interface de parametros de configuracao de Nodes.
 * @author DES FP FKP RVC
 * @author $Author: dseki $
 * @version $Revision: 1.8 $
 */

public interface ConfigurationNode
{
    /**
     * Le do objeto de configuracoes o numero maximo de conexoes da
     * Peer-To-Peer Headers Network.
     *
     * @return numero maximo de conexoes da P2PHN
     */
    public int getP2PMaxConnections();

    /**
     * Le do objeto de configuracoes o numero maximo de nodes a serem
     * armazenados visando novas conexoes na rede P2PHN.
     *
     * @return numero maximo de nodes conhecidos para guardar
     */
    public int getP2PMaxSeenNodes();

    /**
     * Le do objeto de configuracoes o Time-To-Live ("Tempo de Vida" ou nivel
     * de recursao) de um cabecalho (Header) na P2PHN.
     *
     * @return tempo de vida de um Header
     */
    public short getP2PTimeToLive();

    /**
     * Le do objeto de configuracao o diretorio padrao onde serao armazenados
     * os novos arquivos capturados pelo programa.
     *
     * @return string com o diretorio padrao para salvar os arquivos
     */
    public String getTreecastDefaultDirectory();

    /**
     * Le do objeto de configuraco o numero maximo de conexoes da Treecast
     * Network (TN).
     *
     * @return numero maximo de conexoes da TN
     */
    public int getTreecastMaxConnections();

    /**
     * Le do objeto de configuracao o tempo em segundos de espera antes de
     * parar um Push, ou seja, evitar que novas transmissoes sejam criadas para
     * ele.
     *
     * @return tempo em segundos de espera antes de parar push
     */
    public long getTreecastTimeout();

    /**
     * Le a arvore de filtros do arquivo de configuracoes.
     *
     * @return arvore de filtro
     */
    public FilterRoot getFilterTree();
}
