package tcbnet.control;

import tcbnet.corba.Header;

/**
 * Filtro de campos de texto. Procura dentro dos campos de texto por um texto
 * definido.
 *
 * @author $Author: fabipiesigilli $
 * @version $Revision: 1.15 $
 */

public class FilterText
    extends FilterNode
{
    private String word;

    /**
     * Construtor. Cria um filtro vazio.
     *
     * @param word texto que deve ser encontrado nos campos de texto
     */
    public FilterText(String word)
	throws IllegalArgumentException
    {
	super();
	setWord(word);
    }

    /**
     * Construtor. Cria um filtro vazio.
     *
     * @param word texto que deve ser encontrado nos campos de texto
     */
    public FilterText()
    {
	super();
    }


    /**
     * Define o texto a ser procurado por este filtro.
     *
     * @param word texto a ser procurado
     * @throws IllegalArgumentException se o texto for nulo, vazio ou espacos
     */
    public void setWord(String word)
	throws IllegalArgumentException
    {
	if (word == null || word.trim().equals(""))
	    throw new IllegalArgumentException();
	this.word = word.trim();
    }

    /**
     * Dado um header, decide se ele passa no filtro ou nao.
     * Passa pelo filtro se a descricao ou o nome do arquivo contem a frase de
     * filtragem especificada.
     *
     * @param  h head a ser avaliado
     * @return se o arquivo representado pelo cabecalho deve ser salvo ou nao
     */
    public synchronized boolean passFilter(Header h)
    {
	if (   h.fileDesc.indexOf(this.word) == -1
	    && h.fileName.indexOf(this.word) == -1)
	    return false;
	return true;
    }

    /**
     * Retorna uma representacao do filtro de tamanho de arquivo no formato de
     * texto.
     *
     * @return representacao no formato texto do filtro
     */
    public String toString() {
	return "Texto : " + word;
    }

    public Object clone() {
	Object copy = null;
        copy = super.clone();
	// PRECISA DISTO?
	((FilterText)copy).setWord(word);
	return copy;
    }

}


