package tcbnet.corba;

import org.omg.PortableServer.POA;
import org.omg.PortableServer.POAHelper;
import java.util.*;

/**
 * <p>Title: TCBnet</p>
 * <p>Description: p2p content push</p>
 * <p>Copyright: TCB inc Copyright (c) 2002</p>
 * <p>Company: TCB inc</p>
 * @author $Author: rcouto $
 * @version $Revision: 1.5 $
 */
public class TcbnetOrb {
    private static Thread threadRun = null;

    private static TcbnetOrb tcbnetorb = null;
    private static Object instanciatedCond = new Object ();;

    private static boolean running = false;
    private static Object runningCond = new Object ();
    
    private org.omg.CORBA.ORB orb;
    private POA poa;

  /**
   * Cria e inicializa a orb por meio de uma thread.
   * @param args os argumentos a serem passados para o init do ORB
   */
  private TcbnetOrb(String[] args) {
      try {
	  orb = org.omg.CORBA.ORB.init (args, null);
	  org.omg.CORBA.Object obj =
	      orb.resolve_initial_references ("RootPOA");
	  poa = POAHelper.narrow (obj);
      } catch (Exception e) {
	  e.printStackTrace();
	  System.exit(0);
      }
      this.run();
  }

  /**
   * Inicializa o ORB
   */
  private void run() {
    this.threadRun = new Thread() {
      public void run() {
        try {
          poa.the_POAManager ().activate ();
          System.out.println("Servidor Ativado");
	  TcbnetOrb.running = true;
	  synchronized (TcbnetOrb.runningCond) {
	      TcbnetOrb.runningCond.notify ();
	  }
          orb.run ();
        } catch (Exception e) {
          e.printStackTrace();
        }
      }
    };
    this.threadRun.start();
    this.running = true;
  }

  /**
   * Pega a instncia de TcbnetOrb ativa (Singleton).
   * @return TcbnetOrb
   */
  public static TcbnetOrb getInstance() {
      synchronized (TcbnetOrb.instanciatedCond) {
	  if (TcbnetOrb.tcbnetorb == null)
	      TcbnetOrb.tcbnetorb = new TcbnetOrb(null);  
      }
      synchronized (TcbnetOrb.runningCond) {
	  while (! TcbnetOrb.running) {
	      try { TcbnetOrb.runningCond.wait (); }
	      catch (InterruptedException ie) {
		  /** @todo rever */
		  System.exit (1);
	      }
	  }
      }
      return TcbnetOrb.tcbnetorb;
  }

  /**
   * Devolve o objeto remoto Node a partir da sua IOR
   * @param nodeIor a ior do objeto n
   * @return o objeto remoto n
   */
  public Node getNode(String nodeIor) {
    org.omg.CORBA.Object obj = orb.string_to_object(nodeIor);
    return NodeHelper.narrow(obj);
  }

  /**
   * Devolve o objeto remoto Node a partir de sua implementao
   * @param nodeImpl a implementao do n
   * @return o objeto remoto n
   */
  public Node getNode(NodeImpl nodeImpl) {
    return nodeImpl._this(orb);
  }

  /**
   * Devolve o objeto remoto Stream a partir de sua IOR
   * @param streamIor a ior do objeto stream
   * @return o objeto remoto stream
   */
  public Stream getStream(String streamIor) {
    org.omg.CORBA.Object obj = orb.string_to_object(streamIor);
    return StreamHelper.narrow(obj);
  }

  /**
   * Devolve o objeto remoto Stream a apartir de sua implementao
   * @param streamImpl a implementao da stream
   * @return o objeto remoto stream
   */
  public Stream getStream(StreamImpl streamImpl) {
    return streamImpl._this(orb);
  }

  /**
   * Devolve a ior que o identifica o objeto passado
   * @param obj o objeto
   * @return a ior gerada
   */
  public String getIor(org.omg.PortableServer.Servant obj) {
    return orb.object_to_string(obj._this_object(orb));
  }

  /**
   * Funao de auto-teste
   * @param args
   */
  public static void main(String[] args) {
    TcbnetOrb orb = TcbnetOrb.getInstance();
  }

}
