package tcbnet.io;

import java.io.RandomAccessFile;
import java.io.IOException;
import java.io.FileNotFoundException;
import java.util.zip.Adler32;
import tcbnet.corba.Header;
import tcbnet.corba.NodeImpl;
import tcbnet.corba.Node;

/**
 * <p>Title: TCBnet</p>
 * <p>Description: p2p content push</p>
 * <p>Copyright: TCB inc Copyright (c) 2002</p>
 * <p>Company: TCB inc</p>
 * @author $Author: fernandopeinado $
 * @version $Revision: 1.10 $
 */

public class FileHeaderImpl implements FileHeader {
  private static final int CS_BUFFER_SIZE = 1024;
  private RandomAccessFile file;
  private String fileName;
  private long fileSize;
  private String MIMEType;
  private String checkSum;

  /**
   * Constroi um Header com as informaes bsicas do arquivo dado
   *
   * @param fileName o nome do arquivo
   * @param desc a descrio do arquivo
   * @return Header com as informaes bsicas
   * @exception FileNotFoundException caso o arquivo no exista
   * @exception IOException caso ocorra algum erro de IO
   */
  public Header getHeaderFromFile(String fileName, String desc)
    throws FileNotFoundException, IOException {
    this.fileName = fileName;

    file = new RandomAccessFile(this.fileName, "r");
    this.fileSize = file.length();
    this.MIMEType = getMIMEType();
    this.checkSum = getCheckSum();

    Header h = new Header();
    h.fileName = this.fileName;
    h.fileSize = this.fileSize;
    h.MIMEType = this.MIMEType;
    h.checkSum = this.checkSum;
    h.fileDesc = desc;
    h.visited  = new Node[1]; /** @todo falta decidir aqui o que fazer */
    return h;
  }

  /**
   * Descobre qual o MIMEType do arquivo e retorna-o como uma string
   * @return o MIMEType do arquivo
   */
  private String getMIMEType() {
    /**
     * @todo pesquisar como fazer e preparar para que esta funo se baseie em
     * um xml para decidir o MIME type de um arquivo pela sua extenso
     */
    return "unknow";
  }

  /**
   * Calcula a soma de verificao atravs da classe Adler32 tambm utilizada
   * pelo compactador ZIP
   * @return a soma de verificao
   * @throws IOException
   */
  private String getCheckSum() throws IOException {
    Adler32 cs = new Adler32();
    byte[] in = new byte[this.CS_BUFFER_SIZE];

    while(file.read(in) != -1) {
      cs.update(in);
    }

    return "" + cs.getValue();
  }
}
