package tcbnet.io;

import java.io.RandomAccessFile;
import java.io.FileNotFoundException;
import java.io.IOException;

/**
 * <p>Title: TCBnet</p>
 * <p>Description: p2p content push</p>
 * <p>Copyright: TCB inc Copyright (c) 2002</p>
 * <p>Company: TCB inc</p>
 * @author $Author: fernandopeinado $
 * @version $Revision: 1.7 $
 */

public class FileReaderImpl implements FileReader {
  /** tamanho padro do buffer 1 kb */
  static final int DEFAULT_BUFFER_SIZE = 1024;
  //  static final int DEFAULT_BUFFER_SIZE = 1;

  /** o arquivo de a ser lido */
  protected RandomAccessFile file;
  /** o tamanho do buffer em bytes */
  protected int bufferSize;

  /**
   * Construtor Padro.
   * Define o tamanho do buffer com o tamanho default.
   */
  public FileReaderImpl() {
    this.bufferSize = FileReaderImpl.DEFAULT_BUFFER_SIZE;
  }

  /**
   * Abre o arquivo pelo seu nome com permisso de leitura
   * @param fileName o nome do arquivo
   * @throws FileNotFoundException
   */
  public void openFile(String fileName) throws FileNotFoundException {
    this.file = new RandomAccessFile(fileName, "r");
  }

  /**
   * Define o tamanho do buffer de transmisso de arquivo.
   * @param size o tamanho do buffer em bytes
   */
  public void setBufferSize(int size) {
    this.bufferSize = size;
  }

  /**
   * Devolve o tamanho do arquivo em bytes
   * @return long fileSize
   */
  public long getFileSize() throws IOException {
    if (this.file != null) return this.file.length();
    else return 0;
  }

  /**
   * Carrega e retorna um buffer com um segmento do arquivo
   * @return byte[] contendo o buffer lido, ou null caso no haja mais
   *         nada para ser lido
   * @throws IOException
   */
  public byte[] readBuffer() throws IOException {
    byte[] buffer = new byte[this.bufferSize];
    int readed = this.file.read(buffer);
    System.out.println("size: " + this.file.length() + " lido: " + buffer.length + "  " + readed);

    if (readed != -1)
      return buffer;
    else
      return null;
  }

  /**
   * Reinicia a transmisso do arquivo
   */
  public void resetFile() throws IOException {
    this.file.seek(0);
  }

  /**
   * Reinicia a transmisso do arquivo para uma posio escolhida
   * @param pos a posio escolhida
   */
  public void resetFile(long pos) throws IOException {
    this.file.seek(pos);
  }

  /**
   * define se o arquivo chegou ao fim
   * @return true ou false
   */
  public boolean eof() throws IOException {
    try {
      file.getFilePointer();
    }
    catch (java.io.EOFException ex) {
      return true;
    }
    catch (IOException ex) {
      throw ex;
    }
    return false;
  }
}
