package tcbnet.treecast;

import tcbnet.corba.*;
import tcbnet.util.*;

/**
 * Classe responsavel para manter o programa conectado a rede P2PHN.
 *
 * @author $Author: rcouto $
 * @version $Revision: 1.25 $
 */
public class P2PKeepAlive extends Thread {
    private Node localNode;
    private P2PNodeList nodeList;
    private int maxConnections;
    private ProdConsMonitor nodeBuffer;

    /**
     * Construtor. Inicializa thread.
     *
     * @param local Referencia para o Node local.
     * @param nl Lista com os Nodes conectados.
     * @param mc Numero maximo de conexoes permitidas.
     */
    public P2PKeepAlive (Node local, P2PNodeList nl, int mc) {
	this.localNode = local;
	this.nodeList = nl;
	this.maxConnections = mc;
	this.nodeBuffer = new ProdConsMonitor ();
	this.start ();
    }

    /**
     * Mantem programa conectado ao numero maximo de Nodes possiveis. Caso
     * atinja esse estado, dorme. O <b>Announce</b> e bidirecional.
     */
    public void run () {
	while (true) {
	    /** @todo TIRAR A PORRA DA ESPERA OCUPADA */
	    if (this.nodeList.size() < this.maxConnections) {
		/*		if (this.nodeBuffer.isEmpty ()) {
		    this.addAll (this.nodeList.getLastSeenNodes ());
		    if (this.nodeBuffer.isEmpty ()) continue;
		    }*/
		Node n = (Node) this.nodeBuffer.take ();
		try {
		    n.announce (this.localNode);
		    this.nodeList.add (n);
		}
		catch (NoConnectionAvailableException ncae) {
		    System.out.println ("SEM ESPACO QUERO FILHOS");
		    System.exit (1);
		    this.nodeBuffer.putAll (ncae.children);
		}
	    }
	}
    }

    /**
     * Configura numero maximo de conexoes.
     *
     * @param mc Novo numero maximo de conexoes permitidas.
     */
    public synchronized void setMaxConnections (int mc) {
	this.maxConnections = mc;
    }

    /**
     * Adiciona um Array de Nodes na fila de Nodes possiveis de se conectar.
     *
     * @param n Array de Nodes possiveis de se conectar.
     */
    public synchronized void addAll (Node[] n) {
	this.nodeBuffer.putAll (n);
    }

    /**
     * Adiciona um Node na fila de Nodes possiveis de se conectar.
     *
     * @param n Node possivel de se conectar.
     */
    public synchronized void add (Node n) {
	this.nodeBuffer.put (n);
    }

}
