package tcbnet.treecast;

import java.io.FileNotFoundException;
import java.util.*;
import tcbnet.corba.*;
import tcbnet.io.*;

/**
 * Classe responsavel por controlar criacao de Streams.
 *
 * @author $Author: fernandopeinado $
 * @version $Revision: 1.19 $
 */
public class StreamControl {

    private int maxStreams;
    private Collection streams;

    /**
     * Construtor.
     *
     * @param ms Numero maximo de Streams simultaneas.
     */
    public StreamControl (int ms) {
	this.maxStreams = ms;
	this.streams = new ArrayList ();
    }

    /**
     * Retorna uma nova Stream que transmitira o arquivo descrito pelo Header.
     *
     * @param h Header que descreve arquivo a ser transmitido.
     * @param dir Diretorio onde se encontra tal arquivo.
     * @return Stream que transmitira o arquivo ou null em caso de falha.
     * @throws NotAvailableException
     */
    public synchronized Stream getStream (Header h, String dir)
	throws NotAvailableException {
	FileReader fr = null;
	System.out.println("getStream : StreamControl");
	if (this.streams.size () < this.maxStreams) {
	    try {
		fr = new FileReaderImpl ();
		fr.openFile (dir.concat (h.fileName));
	    }
	    catch (FileNotFoundException fnfe) {
		throw new NotAvailableException ();
	    }
	    Stream s = Factory.createStream (fr);
	    this.streams.add (s);
	    return s;
	}
	return null;
    }

    /**
     * Retorna uma nova Stream que transmitira o arquivo descrito pelo Header,
     * com a diferenca que esta Stream pode mandar um arquivo enquanto ele
     * ainda esta sendo enviado por outra Stream.
     *
     * @param h Header que descreve arquivo a ser transmitido.
     * @param dir Diretorio onde se encontra tal arquivo.
     * @return Stream que transmitira o arquivo ou null em caso de falha.
     * @throws NotAvailableException
     */
    public synchronized Stream getStreamReceiver (Header h, String dir)
	throws NotAvailableException {
	System.out.println("getStreamReceiver : StreamControl");
	if (this.streams.size () < this.maxStreams) {
	    FileReaderTail fr = null;
	    try {
		fr = new FileReaderTail ();
		fr.setOriginalSize (h.fileSize);
		fr.openFile (dir.concat (h.fileName));
	    }
	    catch (FileNotFoundException fnfe) {
		throw new NotAvailableException ();
	    }
	    Stream s = Factory.createStream (fr);
	    this.streams.add (s);
	    return s;
	}
	return null;
    }

}
