package tcbnet.treecast;

import java.io.IOException;
import java.io.FileNotFoundException;
import tcbnet.corba.*;
import tcbnet.io.*;
import tcbnet.util.*;

/**
 * Responsavel pela retransmissao de um arquivo num Treecast.
 *
 * @author $Author: rcouto $
 * @version $Revision: 1.39 $
 */
public class TreecastReceiver extends TreecastSender {

    private TreecastDownloader treecastDownloader;

    /**
     * Construtor, inicia um novo TreecastDownloader.
     *
     * @param sc StreamControl para criar novas streams.
     * @param sd Diretorio onde sera lido o arquivo.
     * @param h Header do arquivo a ser retransmitido.
     * @param ln Node local.
     */
    public TreecastReceiver (StreamControl sc, String sd, Header h, Node ln)
	throws FileNotFoundException {
	/* Isso soh eh possivel por que construtores sao "sincronizados": ate
	 * ele retornar, ninguem tem referencia a mim, portanto, nao podem
	 * fazer chamados de metodos. */
	super (sc, sd, h);
	Stream s = this.getRemoteStream (h, ln);
	FileWriter fw = new FileWriterImpl ();

	fw.openFile (sd.concat (h.fileName));
	this.treecastDownloader = new TreecastDownloader (s, fw);
    }

    /**
     * Retorna uma nova Stream para o arquivo transmitido nesse Treecast.
     *
     * @return Stream nova ou null em caso de erro
     * @throws NotAvailableException
     */
    protected Stream getNewStream () throws NotAvailableException {
	return this.streamControl.getStreamReceiver (this.header,
						     this.dirPath);
    }

    /**
     * Tenta conseguir uma Stream do Treecast descrito no Header.
     *
     * @param h Header que descreve treecast ao qual quero me conectar.
     * @param ln Node local.
     */
    private Stream getRemoteStream (Header h, Node ln) {
	Node n;
	Stream s = null;
	Queue nodeQueue = new Queue ();
	nodeQueue.add (h.node);
	while ((n = (Node) nodeQueue.remove ()) != null) {
	    try {
		s = n.getStream (ln, h.checkSum);
		break;
	    }
	    catch (NoStreamAvailableException nsae) {
		nodeQueue.addAll (nsae.children);
		System.out.println("ERRO NO STREAM AVAILABLE");
	    }
	    catch (NotAvailableException nae) {
		System.out.println("ERRO NOT AVAILABLE");
		/** @todo gerar excecao */
	    }
	}
	/** @todo se nao conseguiu, gerar uma excecao */
	return s;
    }

    /**
     * @return o quanto do arquivo ja foi transferido pelo downloader.
     */
    public long getTransfered () {
	return this.treecastDownloader.getTransfered ();
    }
}

/**
 * Classe responsavel por salvar o arquivo transmitido num Treecast.
 *
 * @author $Author: rcouto $
 * @version $Revision: 1.39 $
 */
class TreecastDownloader extends Thread {

    private Stream stream;
    private FileWriter fileWriter;
    private long transfered = 0;

    /**
     * Construtor.
     *
     * @param s Stream de onde ira ler o arquivo.
     * @param fw Arquivo onde ira salvar o arquivo.
     */
    public TreecastDownloader (Stream s, FileWriter fw) {
	this.stream = s;
	this.fileWriter = fw;
	System.out.println("Iniciando o Download");
	this.start();
    }

    /**
     * Enquanto nao chegou ao final da Stream, le dela e salva no arquivo
     * local.
     */
    public void run () {
	byte[] buffer = null;

	while (true) {
	    if (this.stream == null) { System.out.println("stream null"); }
	    try {
		BytesHolder bh = new BytesHolder();
		bh.value = new byte[1];
		int eof = this.stream.getBuffer (bh);
		if (eof == -1) buffer = null;
		else buffer = bh.value;
	    }
	    catch (IOErrorException ioee) {
		System.out.println("ERRO COM LEITURA");
		/** @todo implementar handler da excecao */
	    }
	    System.out.print("#");
	    try {
		if (buffer == null) {
		    System.out.println("Acabou o Download");
		    fileWriter.closeFile();
		    break;
		}
	    } catch (IOException ioe) {
		System.out.println("ERRO AO FECHAR O ARQUIVO");
	    }
	    try {
		this.fileWriter.writeBuffer (buffer);
		this.transfered += buffer.length;
	    }
	    catch (IOException ioe) {
		System.out.println("ERRO COM ESCRITA");
		/** @todo implementar handler da excecao */
	    }
	}
    }

    /**
     * @return o quanto do arquivo ja foi transferido pelo downloader.
     */
    public long getTransfered () {
	return this.transfered;
    }    
}



