package tcbnet.treecast;

import java.io.*;
import java.util.*;
import tcbnet.corba.*;
import tcbnet.io.*;

/**
 * Classe responsavel por transmitir um arquivo local num Treecast.
 *
 * @author $Author: rcouto $
 * @version $Revision: 1.35 $
 */
public class TreecastSender implements Treecast {
    
    protected StreamControl streamControl;
    protected String dirPath;
    protected Header header;
    protected Map streams;
    protected boolean active;
    
    /**
     * Construtor.
     *
     * @param sc StreamControl para gerar novos Streams.
     * @param to Timeout em milisegundos para o Treecast ser desativado.
     * @param dir Diretorio onde encontrarei arquivo a ser enviado.
     * @param h Header que descreve arquivo a ser enviado.
     */
    public TreecastSender (StreamControl sc, String dir, Header h) {
	this.streamControl = sc;
	this.dirPath = dir;
	this.header = h;
	this.streams = new HashMap ();
	this.active = true;
    }

    /**
     * Comeca aceitar pedidos de Stream
     */
    public synchronized void start () {
	this.active = true;
    }

    /**
     * Para de aceitar pedidos de Stream.
     */
    public synchronized void stop () {
	this.active = false;
    }

    /**
     * Retorna uma Stream para que um Node filho possa receber o arquivo.
     *
     * @param n Node que quer conseguir uma Stream.
     * @return Uma Stream que transmitira o arquivo.
     * @throws NoStreamAvailableException
     * @throws NotAvailableException
     */
    public synchronized Stream getStream (Node n)
	throws NoStreamAvailableException, NotAvailableException {
	System.out.println("getStream : TreecastSender");
	if (this.active) {
	    Stream s = this.getNewStream ();
	    if (s == null)
		throw new NoStreamAvailableException (this.getChildNodes ());
	    this.addStream (n, s);
	    return s;
	}
	else
	    throw new NotAvailableException ();
    }

    /**
     * Adiciona o Stream na lista de Streams ativos.
     *
     * @param s Stream a ser adicionado.
     */
    protected void addStream (Node n, Stream s) {
	this.streams.put (n, s);
    }

    /**
     * Retorna uma nova Stream para o arquivo transmitido nesse Treecast.
     *
     * @return Stream nova ou null em caso de erro
     * @throws NotAvailableException
     */
    protected Stream getNewStream () throws NotAvailableException {
	return this.streamControl.getStream (this.header, this.dirPath);
    }

    /**
     * Retorna uma lista com os Nodes recebendo o Treecast.
     *
     * @return Lista com os Nodes recebendo o Treecast.
     */
    protected Node[] getChildNodes () {
	Set ks = this.streams.keySet ();
	return (Node[]) ks.toArray ();
    }

    /**
     * @return o Header que o Treecast esta transmitindo.
     */
    public Header getHeader () {
	return this.header;
    }
    
    /**
     * @return o numero de Streams ativas do Treecast.
     */
    public int getStreamCount () {
	return this.streams.size ();
    }

    /**
     * @return o numero de Streams jah completadas do Treecast.
     */
    public int getStreamDoneCount () {
	/** @todo arrumar isso */
	return 0;
    }
}



