package tcbnet.ui;

import tcbnet.control.*;
import java.io.Serializable;

/**
 * Processa, armazena, salva e carrega parametros de configuracao da interface
 * com o usuario (UI - User Interface).
 *
 * @author $Author: dseki $
 * @version $Revision: 1.9 $
 */

public class UIConfiguration
    implements Serializable
{
    private PushQueue pushQueue;
    private int pushQueueMaxPushes;

    /**
     * Construtor da classe.
     */
    public UIConfiguration()
    {
	this.pushQueue = new PushQueue();
	this.pushQueueMaxPushes = 2;
    }

    /**
     * Le a fila de push do objeto de configuracoes.
     *
     * @return conteudo da fila de push
     */
    public synchronized PushQueue getPushQueue()
    {
	return this.pushQueue;
    }

    /**
     * Salva o estado da fila de push no objeto de configuracoes.
     *
     * @param pushQueue fila de push a ser salva
     */
    public synchronized void setPushQueue(PushQueue pushQueue)
    {
	this.pushQueue = pushQueue;
    }

    /**
     * Devolve o numero maximo de arquivos disponiveis para transmissao
     * simultaneamente.
     *
     * @return numero maximo de broadcasts simultaneos
     */
    public synchronized int getPushQueueMaxPushes()
    {
	return this.pushQueueMaxPushes;
    }

    /**
     * Define o numero maximo de arquivos disponiveis para transmissao
     * simultaneamente.
     *
     * @param pushQueueMaxPushes numero maximo de broadcasts simultaneos
     */
    public synchronized void setPushQueueMaxPushes(int pushQueueMaxPushes)
    {
	this.pushQueueMaxPushes = pushQueueMaxPushes;
    }
}
